// Copyright 2000-2023 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package org.jetbrains.plugins.terminal.exp

import com.intellij.openapi.Disposable
import java.util.concurrent.TimeUnit

class ZshCompletionManager(session: TerminalSession) : BaseShCompletionManager(session) {
  override fun doResetPrompt(disposable: Disposable) {
    val model = session.model
    val promptRestoredFuture = checkTerminalContent(disposable) {
      model.cursorX == 0
    }
    // Send Ctrl+U to clear the line with typings
    val resetCommand = "\u0015"
    session.terminalStarter.sendString(resetCommand, false)

    promptRestoredFuture.get(3000, TimeUnit.MILLISECONDS)

    model.withContentLock {
      model.clearAllExceptPrompt(1)
    }
  }
}